/*
 * Ǝ@vO
 * Copyright (C) 2007 ~V Ўu
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * ̃vO̓t[\tgEFAłBȂ͂At[\tgEF
 * AcɂĔsꂽ GNU ʌOp_񏑃o[W2̒߂
 * ̉ōĔЕz܂͉ς邱Ƃł܂B
 * 
 * ̃vO͗Lpł邱ƂĔЕz܂A*S̖ۏ* 
 * łBƉ\̕ۏ؂̖ړIւ̓ḰAOɎꂽ̂
 * ߑS݂܂BڂGNU ʌOp_񏑂B
 * 
 * Ȃ͂̃vOƋɁAGNU ʌOp_񏑂̕ꕔ
 * 󂯎͂łB󂯎ĂȂ΁At[\tgEFAc
 * ŐĂ( the Free Software Foundation, Inc., 59
 * Temple Place, Suite 330, Boston, MA 02111-1307 USA)B
 */
package ume.puzzle.sudoku.context;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import ume.puzzle.exception.ContradictionException;

public class Box {
	private int bitmap;
	private boolean dirty;
	private final ArrayList<BoxListener> listeners = new ArrayList<BoxListener>();
	private final Set<BoxGroup> associatedGroups = new HashSet<BoxGroup>();

	public Box(int bitmap) {
		this.bitmap = bitmap;
		dirty = true;
	}

	public int getBitmap() {
		return bitmap;
	}

	public boolean andBitmap(int anotherBitmap) throws ContradictionException {
		int newBitmap;

		newBitmap = bitmap & anotherBitmap;
		if (newBitmap == 0) {
			throw new ContradictionException();
		} else if (newBitmap != bitmap) {
			bitmap = newBitmap;
			dirty = true;
			fireValueChanged();
			if (isNumberFixed()) {
				fireValueFixed();
			}
			return true;
		} else {
			return false;
		}
	}

	public boolean clearBitmap(int anotherBitmap) throws ContradictionException {
		return andBitmap(~anotherBitmap);
	}

	public boolean isDirty() {
		return dirty;
	}

	public void setDirty(boolean dirty) {
		this.dirty = dirty;
	}

	public void associate(BoxGroup group) {
		group.add(this);
		associatedGroups.add(group);
	}

	public Set<BoxGroup> getAssociatedGroups() {
		return Collections.unmodifiableSet(associatedGroups);
	}

	public static int getBitmapFromNumber(int num) {
		return 1 << (num - 1);
	}

	public int getNumber() {
		switch(bitmap) {
		case 0x00000001: return  1;
		case 0x00000002: return  2;
		case 0x00000004: return  3;
		case 0x00000008: return  4;
		case 0x00000010: return  5;
		case 0x00000020: return  6;
		case 0x00000040: return  7;
		case 0x00000080: return  8;
		case 0x00000100: return  9;
		case 0x00000200: return 10;
		case 0x00000400: return 11;
		case 0x00000800: return 12;
		case 0x00001000: return 13;
		case 0x00002000: return 14;
		case 0x00004000: return 15;
		case 0x00008000: return 16;
		case 0x00010000: return 17;
		case 0x00020000: return 18;
		case 0x00040000: return 19;
		case 0x00080000: return 20;
		case 0x00100000: return 21;
		case 0x00200000: return 22;
		case 0x00400000: return 23;
		case 0x00800000: return 24;
		case 0x01000000: return 25;
		case 0x02000000: return 26;
		case 0x04000000: return 27;
		case 0x08000000: return 28;
		case 0x10000000: return 29;
		case 0x20000000: return 30;
		case 0x40000000: return 31;
		default:         return -1;
		}
	}

	public boolean isNumberFixed() {
		return (getNumber() != -1);
	}


	public void addBoxListener(BoxListener listener) {
		listeners.add(listener);
	}

	public void removeBoxListener(BoxListener listener) {
		listeners.remove(listener);
	}

	private void fireValueChanged() {
		for (BoxListener listener: listeners) {
			listener.valueChanged(this);
		}
	}

	private void fireValueFixed() {
		// BoxListener  removeBoxListener Ă悤ɁA炩 listener ̃Rs[Ă
		ArrayList<BoxListener> listenersCopy = new ArrayList<BoxListener>(listeners);
		for (BoxListener listener: listenersCopy) {
			listener.valueFixed(this);
		}
	}
}
