/*
 * Ǝ@vO
 * Copyright (C) 2007 ~V Ўu
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * ̃vO̓t[\tgEFAłBȂ͂At[\tgEF
 * AcɂĔsꂽ GNU ʌOp_񏑃o[W2̒߂
 * ̉ōĔЕz܂͉ς邱Ƃł܂B
 * 
 * ̃vO͗Lpł邱ƂĔЕz܂A*S̖ۏ* 
 * łBƉ\̕ۏ؂̖ړIւ̓ḰAOɎꂽ̂
 * ߑS݂܂BڂGNU ʌOp_񏑂B
 * 
 * Ȃ͂̃vOƋɁAGNU ʌOp_񏑂̕ꕔ
 * 󂯎͂łB󂯎ĂȂ΁At[\tgEFAc
 * ŐĂ( the Free Software Foundation, Inc., 59
 * Temple Place, Suite 330, Boston, MA 02111-1307 USA)B
 */
package ume.puzzle.sudoku.context;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import ume.puzzle.sudoku.context.ContextListener;
import ume.puzzle.sudoku.problem.Problem;

public class Context implements BoxListener {
	private final Problem problem;
	private final ArrayList<ContextListener> listeners = new ArrayList<ContextListener>();
	private final Box[][] boxes;
	private final Set<BoxGroup> groups = new HashSet<BoxGroup>();
	private final Map<Box, Position> boxPosition = new HashMap<Box, Position>();
	private final Set<BoxGroup> unmodifiableGroups = Collections.unmodifiableSet(groups);

	private class Position {
		public final int x;
		public final int y;

		public Position(int x, int y) {
			this.x = x;
			this.y = y;
		}
	}

	public Context(Problem problem) {
		this.problem = problem;

		int blankNumberBitmap = (1 << problem.getMaxNumber()) - 1;
		boxes = new Box[getHeight()][getWidth()];
		for (int y = 0; y < getHeight(); y++) {
			for (int x = 0; x < getWidth(); x++) {
				if (problem.getNumber(x, y) == 0) {
					boxes[y][x] = new Box(blankNumberBitmap);
				} else {
					boxes[y][x] = new Box(Box.getBitmapFromNumber(problem.getNumber(x, y)));
				}
				boxes[y][x].addBoxListener(this);
				boxPosition.put(boxes[y][x], new Position(x, y));
			}
		}

		for (int y = 0; y < getHeight(); y++) {
			BoxGroup group = new BoxGroup(String.format("row %d", y));
			for (int x = 0; x < getWidth(); x++) {
				boxes[y][x].associate(group);
			}
			groups.add(group);
		}

		for (int x = 0; x < getWidth(); x++) {
			BoxGroup group = new BoxGroup(String.format("col %d", x));
			for (int y = 0; y < getHeight(); y++) {
				boxes[y][x].associate(group);
			}
			groups.add(group);
		}

		for (int x = 0; x < getWidth(); x += problem.getBlockWidth()) {
			for (int y = 0; y < getHeight(); y += problem.getBlockHeight()) {
				BoxGroup group = new BoxGroup(String.format("blk %d,%d", x, y));
				for (int dx = 0; dx < problem.getBlockWidth(); dx++) {
					for (int dy = 0; dy < problem.getBlockHeight(); dy++) {
						boxes[y + dy][x + dx].associate(group);
					}
				}
				groups.add(group);
			}
		}
	}

	public int getWidth() {
		return problem.getMaxNumber();
	}

	public int getHeight() {
		return problem.getMaxNumber();
	}

	public Box getBox(int x, int y) {
		return boxes[y][x];
	}

	public Set<BoxGroup> getGroups() {
		return unmodifiableGroups;
	}

	public void valueChanged(Box box) {
	}

	public void valueFixed(Box box) {
		Position pos = boxPosition.get(box);
		fireValueFixed(pos.x, pos.y);
	}

	public void addContextListener(ContextListener listener) {
		listeners.add(listener);
	}

	public void removeContextListener(ContextListener listener) {
		listeners.remove(listener);
	}

	private void fireValueChanged(int x, int y) {
		for (ContextListener listener : listeners) {
			listener.valueChanged(this, x, y);
		}
	}

	private void fireValueFixed(int x, int y) {
		for (ContextListener listener : listeners) {
			listener.valueFixed(this, x, y);
		}
	}

	private void fireCompleted() {
		for (ContextListener listener : listeners) {
			listener.completed(this);
		}
	}
}
